<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\campaignmonitor
 * @copyright Copyright (c) 2013 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * campaignmonitor Class
 *
 * Class for interfacing with CampaignMonitor API.
 *
 * @package clearFusionCMS\module\campaignmonitor
 */
class campaignmonitor {

	/**
	 * The API key.
	 *
	 * @var string
	 */
	protected $apiKey;

	/**
	 * The client Id.
	 *
	 * @var string
	 */
	protected $clientId;

	/**
	 * The last error.
	 *
	 * @var string
	 */
	protected $lastError = '';

	/**
	 * Construct the object.
	 *
	 * @param string $apiKey The API key to use or null to use the system set key.
	 * @param string $clientId The client Id to use or null to use the system set value.
	 */
	function __construct($apiKey = null, $clientId = null) {
		if($apiKey === null) {
			$cfg = clearFusionCMS::getSiteConfig('cfCampaignMonitor');
			$this->apiKey = $cfg->apiKey;
			$this->clientId = $cfg->clientId;
		}
		else {
			$this->apiKey = $apiKey;
			$this->clientId = $clientId;
		}
	}

	/**
	 * Get the last error message string.
	 *
	 * @return string The last error string.
	 */
	function getLastError() {
		return $this->lastError;
	}

	/**
	 * Get the available lists.
	 *
	 * @return array of list information or null on error.
	 */
	function getLists() {
		$client = new flHttpClient("https://api.createsend.com/api/v3/clients/$this->clientId/lists.json");
		$client->setAuth($this->apiKey, '');
		if($client->sendRequest() && $client->getResponseCode() == 200)
			return json_decode($client->getBody());

		return null;
	}

	/**
	 * Get the list of sent campaigns.
	 *
	 * @return array of sent campaigns or null on error.
	 */
	function getSentCampaigns() {
		$client = new flHttpClient("https://api.createsend.com/api/v3/clients/$this->clientId/campaigns.json");
		$client->setAuth($this->apiKey, '');
		if($client->sendRequest() && $client->getResponseCode() == 200)
			return json_decode($client->getBody());

		return null;
	}

	/**
	 * Subscribe a user to a list.
	 *
	 * @param string $listId The ID of the list to subscribe to.
	 * @param string $email The email address to subscribe.
	 * @param string $name The name of the subscriber.
	 * @param array $vars Array of custom fields.
	 * @return bool true if the user was added to the list; else false.
	 */
	function subscribe($listId, $email, $name, $vars) {
		$d = array(
			'EmailAddress' => $email,
			'Name' => $name,
			'Resubscribe' => true,
			'RestartSubscriptionBasedAutoresponders' => true
		);
		if(count($vars))
			$d['CustomFields'] = $vars;

		$client = new flHttpClient("https://api.createsend.com/api/v3/subscribers/$listId.json");
		$client->setAuth($this->apiKey, '')
			->setPostData(json_encode($d));
		if($client->sendRequest() && $client->getResponseCode() == 201)
			return true;
		else
			return false;
	}

	/**
	 * Update a list subscribers email address.
	 *
	 * @param string $listId The ID of the list to subscribe to.
	 * @param string $oldEmail The email address to update.
	 * @param string $newEmail The new email address.
	 * @param string $name The name of the subscriber.
	 * @param array $vars Array of custom fields.
	 * @return bool true if the user was updated; else false.
	 */
	function update($listId, $oldEmail, $newEmail, $name, $vars) {
		$d = array(
			'EmailAddress' => $newEmail,
			'Name' => $name,
			'Resubscribe' => true,
			'RestartSubscriptionBasedAutoresponders' => true
		);
		if(count($vars))
			$d['CustomFields'] = $vars;

		$client = new flHttpClient("https://api.createsend.com/api/v3/subscribers/$listId.json?email=" . urlencode($oldEmail));
		$client->setAuth($this->apiKey, '')
			->setPostData(json_encode($d));
		if($client->sendRequest('PUT') && $client->getResponseCode() == 200)
			return true;
		else
			return false;
	}

	/**
	 * Unsubscribe a user from a list.
	 *
	 * @param string $listId The ID of the list to unsubscribe from.
	 * @param string $email The email address to ubsubscribe.
	 * @return bool true if the user was removed from the list; else false.
	 */
	function unsubscribe($listId, $email) {
		$client = new flHttpClient("https://api.createsend.com/api/v3/subscribers/$listId/unsubscribe.json");
		$client->setAuth($this->apiKey, '');
		$client->setPostData(json_encode(array('EmailAddress' => $email)));
		if($client->sendRequest() && $client->getResponseCode() == 200)
			return true;
		else
			return false;
	}

	/**
	 * Get the custom fields for a list.
	 *
	 * @param string $listId The ID of the list to get the variables for.
	 * @return array List of variables; else null on error.
	 */
	function getFields($listId) {
		$client = new flHttpClient("https://api.createsend.com/api/v3/lists/$listId/customfields.json");
		$client->setAuth($this->apiKey, '');
		if($client->sendRequest() && $client->getResponseCode() == 200)
			return json_decode($client->getBody());
		else
			return false;
	}

	/**
	 * Get the current data for a list subscriber.
	 *
	 * @param string $listId The list ID to query for the subscriber.
	 * @param string $email The email address of the subscriber to query.
	 * @return mixed The subscriber information or false if not found.
	 */
	function getSubscriber($listId, $email) {
		$client = new flHttpClient("https://api.createsend.com/api/v3/subscribers/$listId.json?email=" . urlencode($email));
		$client->setAuth($this->apiKey, '');
		if($client->sendRequest() && $client->getResponseCode() == 200)
			return json_decode($client->getBody());
		else
			return false;
	}
}
