<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\cloudflare
 * @copyright Copyright (c) 2014 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * cloudflareInstall Class
 *
 * Class for managing the CloudFlare module installation.
 *
 * @package clearFusionCMS\module\cloudflare
 */
class cloudflareInstall implements cfInstallInterface {
	/**
	 * The last error message.
	 *
	 * @var string
	 */
	protected $lastError = '';

	/**
	 * Return the last error.
	 *
	 * @return string The last error string.
	 */
	function getLastError() {
		return $this->lastError;
	}

	/**
	 * Install the module.
	 *
	 * @param flDb The database object to use.
	 * @return bool true if installed; else false.
	 */
	function install($db) {

		// Create permissions
		$ac = new flAcMgr($db);
		$ac->createPermissionGroup('cfCloudFlare', __noop('Cloudflare'), 'cloudflareBackend'); // 5000
		$ac->createPermission('cfCloudFlare.access', __noop('Can manage Cloudflare.'), 5010, 'cloudflareBackend');
		$ac->createPermission('cfCloudFlare.settings', __noop('Can edit Cloudflare settings.'), 5010, 'cloudflareBackend');

		// Add to dashboard
		$dmgr = new cfDashboardMgr($db);
		$dmgr->addIcon('cfResources', 'cfCloudFlare.access', 'cfCloudFlare.access', __noop('Cloudflare'), __noop('Manage Cloudflare CDN.'), 'system/modules/cloudflare/images/cloudflare.svg', 'cloudflare', 'dashboard', 'cloudflareBackend');

		$cfg = new flConfigDb('cfCloudFlare', $db);
		$cfg->email = '';
		$cfg->tkn = '';

		// Add listeners
		$em = clearFusionCMS::getEventManager();
		$em->addListener('system.cacheForceFlush', 'cloudflareEvents', 'onCacheFlush');
		$em->addListener('system.cachePurge', 'cloudflareEvents', 'onCachePurge');
		$em->addListener('system.managerGetNotices', 'cloudflareEvents', 'onGetNotices');

		return true;
	}

	/**
	 * Uninstall the module.
	 *
	 * @param flDb The database object to use.
	 * @return bool true if uninstalled; else false.
	 */
	function uninstall($db) {
		// Remove listeners
		$em = clearFusionCMS::getEventManager();
		$em->removeListener('system.cacheForceFlush', 'cloudflareEvents', 'onCacheFlush');
		$em->removeListener('system.cachePurge', 'cloudflareEvents', 'onCachePurge');
		$em->removeListener('system.managerGetNotices', 'cloudflareEvents', 'onGetNotices');

		$cfg = new flConfigDb('cfCloudFlare', $db);
		$cfg->deleteNamespace();

		// Remove from dashboard
		$dmgr = new cfDashboardMgr($db);
		$dmgr->delIcon('cfCloudFlare.access');

		// Remove permissions
		$ac = new flAcMgr($db);
		$ac->deletePermissionGroup('cfCloudFlare');

		// Delete the tables
		$siteMgr = new cfSiteManager($db);
		$sites = $siteMgr->listSites();
		foreach($sites as $site => $n) {
			$this->deleteSite($db, $site);
		}

		// Tell everyone this module has been removed
		clearFusionCMS::getEventManager()->notify(new flEvent('system.moduleRemoved', array(
			'module' => 'cloudflare'
			)));

		return true;
	}

	/**
	 * Upgrade the module.
	 *
	 * @param flDb The database object to use.
	 * @param string $installedVer The installed version.
	 * @return bool true if upgraded; else false.
	 */
	function upgrade($db, $installedVer) {
		if(version_compare($installedVer, '3.0.1') < 0) {
			$dmgr = new cfDashboardMgr($db);
			$dmgr->setIconName('cfCloudFlare.access', __noop('Cloudflare'), 'cloudflareBackend');
			$dmgr->setIconDesc('cfCloudFlare.access', __noop('Manage Cloudflare CDN.'), 'cloudflareBackend');
		}

		if(version_compare($installedVer, '3.0.2') < 0) {
			$dmgr = new cfDashboardMgr($db);
			$dmgr->setIconFile('cfCloudFlare.access', 'system/modules/cloudflare/images/cloudflare.svg');
		}

		return true;
	}

	/**
	 * Create a new site.
	 *
	 * @param flDb $db The database object.
	 * @param string $site The short name of the site.
	 * @return bool true if the site was created; else false.
	 */
	function createSite($db, $site) {
		return true;
	}

	/**
	 * Delete a site.
	 *
	 * @param flDb $db The database object.
	 * @param string $site The name of the site to delete.
	 * @return bool true if the site was deleted; else false.
	 */
	function deleteSite($db, $site) {
		$v = new flVariables('cfCloudFlare', clearFusionCMS::getDb());
		unset($v->$site);
		return true;
	}
}
