<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\hipchat
 * @copyright Copyright (c) 2013 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * hipchatSettings Class
 *
 * Class to implement settings for the HipChat module.
 *
 * @package clearFusionCMS\module\hipchat
 */
class hipchatSettings implements cfModuleSettingsInterface {

	/**
	 * Get a list of settings groups.
	 *
	 * @return array Array of setting groups, name => func name
	 */
	function getSettingsGroups() {
		if(!clearFusionCMS::getUser()->hasPermission('cfHipChat.settings'))
			return array();

		return array(
			__('Settings', 'hipchatBackend') => 'general'
		);
	}

	/**
	 * Implement HipChat settings.
	 *
	 * @param flForm $form The form object to use.
	 * @param flDb $db The database object.
	 * @return bool true if the settings were updated, false if failed to update or null if no change.
	 */
	function generalSetting($form, $db) {
		if(!clearFusionCMS::getUser()->hasPermission('cfHipChat.settings'))
			return null;

		$cfg = clearFusionCMS::getSiteConfig('cfHipChat');

		$token = new flFormElementText($form, 'token', $cfg->token);
		$token->setLabel(__('API Token', 'hipchatBackend'))
			->setDescription(clearFusionCMS::getController()->alertMessage('Info', __('This is available from the HipChat web interface by going to the room then Tokens.', 'hipchatBackend')))
			->setRequired()
			->addValidator(new flFormValidatorRequired());

		$room = new flFormElementText($form, 'room', $cfg->room);
		$room->setLabel(__('Room', 'hipchatBackend'))
			->setDescription(clearFusionCMS::getController()->alertMessage('Info', __('The room name or ID to send messages to, this can be found under rooms within your HipChat account.', 'hipchatBackend')))
			->setRequired()
			->addValidator(new flFormValidatorRequired());

		$level = new flFormElementSelect($form, 'level', $cfg->level, array(100 => __('Disabled', 'hipchatBackend'), 1 => '1', 2 => '2', 3 => '3', 4 => '4', 5 => '5'));
		$level->setLabel(__('Notification Level', 'hipchatBackend'))
			->setDescription(__('Notifications below the selected level will be ignored.', 'hipchatBackend'));

		$notify = new flFormElementSelect($form, 'notify', $cfg->notify, array(100 => __('Disabled', 'hipchatBackend'), 1 => '1', 2 => '2', 3 => '3', 4 => '4', 5 => '5'));
		$notify->setLabel(__('Notify Users at or Above', 'hipchatBackend'))
			->setDescription(__('Notifications at or above the selected level will cause HipChat to notify room participants.', 'hipchatBackend'));

		$submit = new flFormElementSubmit($form, 'update');
		$submit->setCaption(__('Update Settings', 'hipchatBackend'))->setCSS('primary');

		$test = new flFormElementSubmit($form, 'test');
		$test->setCaption(__('Update &amp; Test Settings', 'hipchatBackend'))->setCSS('accent');

		$form->groupElements('submit', array($submit, $test), new flFormDecoratorList(new flFormDecoratorDl()));
		$form->focusElement($token);

		if($form->isSubmitted() && $form->isValid()) {
			cfSettingsAudit::update(
				__('Hipchat', 'hipchatBackend'),
				__('Settings', 'hipchatBackend'),
				[
					'token' => $cfg->get('token'),
					'room' => $cfg->get('room'),
					'level' => $cfg->get('level'),
					'notify' => $cfg->get('notify')
				]
			);

			$cfg->token = $token->getValue();
			$cfg->room = $room->getValue();
			$cfg->level = $level->getValue();
			$cfg->notify = $notify->getValue();

			// Send a test message
			if($test->getValue()) {
				$sink = new hipchatEvents();
				$sink->onNotify(new cfEventNotify(
					$level->getValue(),
					__('HipChat Configuration Test', 'hipchatBackend'),
					sprintf(__('%s has configured the HipChat module, the integration works!', 'hipchatBackend'), clearFusionCMS::getUser()->getUsername())
				));
			}

			return true;
		}

		return null;
	}
}
