<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\mailchimp
 * @copyright Copyright (c) 2013 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * mailchimp Class
 *
 * Class for interfacing with MailChimp API.
 *
 * @package clearFusionCMS\module\mailchimp
 */
class mailchimp {

	/**
	 * The API key.
	 *
	 * @var string
	 */
	protected $apiKey;

	/**
	 * The last error.
	 *
	 * @var string
	 */
	protected $lastError = '';

	/**
	 * Construct the object.
	 *
	 * @param string $apiKey The API key to use or null to use the system set key.
	 */
	function __construct($apiKey = null) {
		if($apiKey === null) {
			$cfg = clearFusionCMS::getSiteConfig('cfMailChimp');
			$this->apiKey = $cfg->apiKey;
		}
		else
			$this->apiKey = $apiKey;
	}

	/**
	 * Get the last error message string.
	 *
	 * @return string The last error string.
	 */
	function getLastError() {
		return $this->lastError;
	}

	/**
	 * Send a request to MailChimp.
	 *
	 * @param string $method The method to invoke.
	 * @param array $data The data for the request.
	 * @return mixed The return data from the request, null if failed.
	 */
	protected function sendRequest($method, $data) {
		list($key, $dc) = explode('-', $this->apiKey);
		$client = new flHttpClient("https://$dc.api.mailchimp.com/1.3/?output=json&method=$method");
		$data['apikey'] = $this->apiKey;
		$client->setPostData($data);
		if($client->sendRequest() && $client->getResponseCode() == 200) {
			$r = json_decode($client->getBody());
			if(isset($r->error)) {
				$this->lastError = $r->error;
				return null;
			}
			else
				return $r;
		}
		else
			return null;
	}

	/**
	 * Get the available lists.
	 *
	 * @return array of list information or null on error.
	 */
	function getLists() {
		return $this->sendRequest('lists', array(
			'start' => 0,
			'limit' => 100
		));
	}

	/**
	 * Subscribe a user to a list.
	 *
	 * @param string $listId The ID of the list to subscribe to.
	 * @param string $email The email address to subscribe.
	 * @param array $vars Array of merge variables.
	 * @param bool $ignoreOptin true to ignore double optin settings.
	 * @return bool true if the user was added to the list; else false.
	 */
	function subscribe($listId, $email, $vars, $ignoreOptin = false) {
		$d = array(
			'id' => $listId,
			'email_address' => $email,
			'update_existing' => 'true'
		);
		if($ignoreOptin)
			$d['double_optin'] = 'false';
		foreach($vars as $k => $v)
			$d["merge_vars[$k]"] = $v;
		return $this->sendRequest('listSubscribe', $d) === true;
	}

	/**
	 * Update a list subscribers email address.
	 *
	 * @param string $listId The ID of the list to subscribe to.
	 * @param string $oldEmail The email address to update.
	 * @param string $newEmail The new email address.
	 * @param array $vars Array of merge variables.
	 * @return bool true if the user was updated; else false.
	 */
	function update($listId, $oldEmail, $newEmail, $vars) {
		$d = array(
			'id' => $listId,
			'email_address' => $oldEmail,
			'merge_vars[NEW-EMAIL]' => $newEmail
		);
		foreach($vars as $k => $v)
			$d["merge_vars[$k]"] = $v;
		return $this->sendRequest('listUpdateMember', $d) === true;
	}

	/**
	 * Unsubscribe a user from a list.
	 *
	 * @param string $listId The ID of the list to unsubscribe from.
	 * @param string $email The email address to ubsubscribe.
	 * @return bool true if the user was removed from the list; else false.
	 */
	function unsubscribe($listId, $email) {
		return $this->sendRequest('listUnsubscribe', array(
			'id' => $listId,
			'email_address' => $email
		)) === true;
	}

	/**
	 * Get the variables for a list.
	 *
	 * @param string $listId The ID of the list to get the variables for.
	 * @return array List of variables; else null on error.
	 */
	function getVariables($listId) {
		return $this->sendRequest('listMergeVars', array('id' => $listId));
	}

	/**
	 * Get the list of sent campaigns.
	 *
	 * @return array List of sent; else null on error.
	 */
	function getSentCampaigns() {
		return $this->sendRequest('campaigns', array('filters[status]' => 'sent'));
	}

	/**
	 * Get the current data for a list subscriber.
	 *
	 * @param string $listId The list ID to query for the subscriber.
	 * @param string $email The email address of the subscriber to query.
	 * @return mixed The subscriber information or null if not found.
	 */
	function getSubscriber($listId, $email) {
		return $this->sendRequest('listMemberInfo', array(
			'id' => $listId,
			'email_address' => $email
		));
	}
}
