<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\mailchimp
 * @copyright Copyright (c) 2013 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * mailchimpSettings Class
 *
 * Class to implement settings for MailChimp.
 *
 * @package clearFusionCMS\module\mailchimp
 */
class mailchimpSettings implements cfModuleSettingsInterface {

	/**
	 * Get a list of settings groups.
	 *
	 * @return array Array of setting groups, name => func name
	 */
	function getSettingsGroups() {
		if(!clearFusionCMS::getUser()->hasPermission('cfMailChimp.settings'))
			return array();

		return array(
			__('API Key', 'mailchimpBackend') => 'apikey'
		);
	}

	/**
	 * Implement API Key settings.
	 *
	 * @param flForm $form The form object to use.
	 * @param flDb $db The database object.
	 * @return bool true if the settings were updated, false if failed to update or null if no change.
	 */
	function apikeySetting($form, $db) {
		if(!clearFusionCMS::getUser()->hasPermission('cfMailChimp.settings'))
			return null;

		$cfg = clearFusionCMS::getSiteConfig('cfMailChimp');

		$apiKey = new flFormElementText($form, 'apikey', $cfg->apiKey);
		$apiKey->setLabel(__('API Key', 'mailchimpBackend'))
			->setRequired()
			->addValidator(new flFormValidatorRequired())
			->setDescription(__('You can find API keys in the MailChimp admin under Account > Extras > API Keys.', 'mailchimpBackend'));

		$avail = new flFormElementPane($form, 'lists', __('Not Available', 'mailchimpBackend'));

		$submit = new flFormElementSubmit($form, 'update');
		$submit->setCaption(__('Update Settings', 'mailchimpBackend'))->setCSS('primary');

		$form->groupElements('keys', array($apiKey))
			->groupElements('lists', array($avail))
			->addTab(__('API Key', 'mailchimpBackend'), 'keys')
			->addTab(__('Lists', 'mailchimpBackend'), 'lists')
			->groupElements('submit', array($submit), new flFormDecoratorList(new flFormDecoratorDl()));

		$form->focusElement($apiKey);

		$r = null;
		if($form->isSubmitted() && $form->isValid()) {
			cfSettingsAudit::update(
				__('MailChimp', 'mailchimpBackend'),
				__('API Key', 'mailchimpBackend'),
				[
					'apiKey' => $cfg->get('apiKey')
				]
			);
			$cfg->apiKey = $apiKey->getValue();
			$r = true;
		}

		// If has an api key then list the available lists
		if(!empty($cfg->apiKey)) {
			$mc = new mailchimp($cfg->apiKey);
			try {
				if($lists = $mc->getLists()) {
					$table = new flWidgetTable();
					$table->setClass('tablewidget bordered striped rowhover')
						->setHeadings(__('ID', 'mailchimpBackend'), __('Name', 'mailchimpBackend'), __('Members', 'mailchimpBackend'))
						->setColData('id', 'name', 'members');

					foreach($lists->data as $l) {
						if($l->visibility == 'pub') {
							$table->addRow(array(
								'id' => $l->id,
								'name' => $l->name,
								'members' => $l->stats->member_count
							));
						}
					}

					$avail->setContent($table->render() . clearFusionCMS::getController()->alertMessage('Info', __('To create a subscribe form use &#91;&#91;!mailchimp.subscribe &amp;listId=`listid`&#93;&#93;<br />To create an unsubscribe form use &#91;&#91;!mailchimp.unsubscribe &amp;listId=`listid`&#93;&#93;<br />Where listid is one of the above list IDs.', 'mailchimpBackend')));
				}
			} catch(Exception $e) {
			}
		}

		return $r;
	}
}
